@@ -1434,46 +1434,49 @@ class ModelStorage(Model):
 
     @dualmethod
     def save(cls, records):
-        if not records:
-            return
-        values = {}
-        save_values = {}
-        to_create = []
-        to_write = []
-        transaction = records[0]._transaction
-        user = records[0]._user
-        context = records[0]._context
-        for record in records:
-            assert transaction == record._transaction
-            assert user == record._user
-            assert context == record._context
-            save_values[record] = record._save_values
-            values[record] = record._values
-            record._values = None
-            if record.id is None or record.id < 0:
-                to_create.append(record)
-            elif save_values[record]:
-                to_write.append(record)
-        transaction = Transaction()
-        try:
-            with transaction.set_current_transaction(transaction), \
-                    transaction.set_user(user), \
-                    transaction.set_context(context):
-                if to_create:
-                    news = cls.create([save_values[r] for r in to_create])
-                    for record, new in izip(to_create, news):
-                        record._ids.remove(record.id)
-                        record._id = new.id
-                        record._ids.append(record.id)
-                if to_write:
-                    cls.write(*sum(
-                            (([r], save_values[r]) for r in to_write), ()))
-        except:
+        while records:
+            latter = []
+            values = {}
+            save_values = {}
+            to_create = []
+            to_write = []
+            transaction = records[0]._transaction
+            user = records[0]._user
+            context = records[0]._context
             for record in records:
-                record._values = values[record]
-            raise
-        for record in records:
-            record._init_values = None
+                if (record._transaction != transaction
+                        or user != record._user
+                        or context != record._context):
+                    latter.append(record)
+                    continue
+                save_values[record] = record._save_values
+                values[record] = record._values
+                record._values = None
+                if record.id is None or record.id < 0:
+                    to_create.append(record)
+                elif save_values[record]:
+                    to_write.append(record)
+            transaction = Transaction()
+            try:
+                with transaction.set_current_transaction(transaction), \
+                        transaction.set_user(user), \
+                        transaction.set_context(context):
+                    if to_create:
+                        news = cls.create([save_values[r] for r in to_create])
+                        for record, new in izip(to_create, news):
+                            record._ids.remove(record.id)
+                            record._id = new.id
+                            record._ids.append(record.id)
+                    if to_write:
+                        cls.write(*sum(
+                                (([r], save_values[r]) for r in to_write), ()))
+            except:
+                for record in to_create + to_write:
+                    record._values = values[record]
+                raise
+            for record in to_create + to_write:
+                record._init_values = None
+            records = latter
 
 
 class EvalEnvironment(dict):
